@@ -567,7 +567,7 @@ update_collapsed (AdwNavigationSplitView *self)
     if (should_focus)
       gtk_widget_grab_focus (focus);
 
-    g_object_remove_weak_pointer (G_OBJECT (focus), (gpointer *) &focus);
+    g_clear_weak_pointer (&focus);
   }
 }
 
@@ -631,6 +631,32 @@ navigation_pop_cb (AdwNavigationSplitView *self)
     gtk_widget_activate_action (parent, "navigation.pop", NULL);
 }
 
+static void
+adw_navigation_split_view_root (GtkWidget *widget)
+{
+  GtkWidget *parent_page;
+
+  GTK_WIDGET_CLASS (adw_navigation_split_view_parent_class)->root (widget);
+
+  parent_page = adw_widget_get_ancestor (widget, ADW_TYPE_NAVIGATION_PAGE, TRUE, TRUE);
+
+  if (parent_page)
+    adw_navigation_page_add_child_nav_split_view (ADW_NAVIGATION_PAGE (parent_page));
+}
+
+static void
+adw_navigation_split_view_unroot (GtkWidget *widget)
+{
+  GtkWidget *parent_page;
+
+  parent_page = adw_widget_get_ancestor (widget, ADW_TYPE_NAVIGATION_PAGE, TRUE, TRUE);
+
+  if (parent_page)
+    adw_navigation_page_remove_child_nav_split_view (ADW_NAVIGATION_PAGE (parent_page));
+
+  GTK_WIDGET_CLASS (adw_navigation_split_view_parent_class)->unroot (widget);
+}
+
 static void
 adw_navigation_split_view_dispose (GObject *object)
 {
@@ -731,8 +757,11 @@ adw_navigation_split_view_class_init (AdwNavigationSplitViewClass *klass)
   object_class->get_property = adw_navigation_split_view_get_property;
   object_class->set_property = adw_navigation_split_view_set_property;
 
+  widget_class->root = adw_navigation_split_view_root;
+  widget_class->unroot = adw_navigation_split_view_unroot;
+
   /**
-   * AdwNavigationSplitView:sidebar: (attributes org.gtk.Property.get=adw_navigation_split_view_get_sidebar org.gtk.Property.set=adw_navigation_split_view_set_sidebar)
+   * AdwNavigationSplitView:sidebar:
    *
    * The sidebar widget.
    *
@@ -744,7 +773,7 @@ adw_navigation_split_view_class_init (AdwNavigationSplitViewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationSplitView:content: (attributes org.gtk.Property.get=adw_navigation_split_view_get_content org.gtk.Property.set=adw_navigation_split_view_set_content)
+   * AdwNavigationSplitView:content:
    *
    * The content widget.
    *
@@ -756,7 +785,7 @@ adw_navigation_split_view_class_init (AdwNavigationSplitViewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationSplitView:collapsed: (attributes org.gtk.Property.get=adw_navigation_split_view_get_collapsed org.gtk.Property.set=adw_navigation_split_view_set_collapsed)
+   * AdwNavigationSplitView:collapsed:
    *
    * Whether the split view is collapsed.
    *
@@ -774,7 +803,7 @@ adw_navigation_split_view_class_init (AdwNavigationSplitViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationSplitView:show-content: (attributes org.gtk.Property.get=adw_navigation_split_view_get_show_content org.gtk.Property.set=adw_navigation_split_view_set_show_content)
+   * AdwNavigationSplitView:show-content:
    *
    * Determines the visible page when collapsed.
    *
@@ -793,7 +822,7 @@ adw_navigation_split_view_class_init (AdwNavigationSplitViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationSplitView:min-sidebar-width: (attributes org.gtk.Property.get=adw_navigation_split_view_get_min_sidebar_width org.gtk.Property.set=adw_navigation_split_view_set_min_sidebar_width)
+   * AdwNavigationSplitView:min-sidebar-width:
    *
    * The minimum sidebar width.
    *
@@ -811,7 +840,7 @@ adw_navigation_split_view_class_init (AdwNavigationSplitViewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationSplitView:max-sidebar-width: (attributes org.gtk.Property.get=adw_navigation_split_view_get_max_sidebar_width org.gtk.Property.set=adw_navigation_split_view_set_max_sidebar_width)
+   * AdwNavigationSplitView:max-sidebar-width:
    *
    * The maximum sidebar width.
    *
@@ -829,7 +858,7 @@ adw_navigation_split_view_class_init (AdwNavigationSplitViewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationSplitView:sidebar-width-fraction: (attributes org.gtk.Property.get=adw_navigation_split_view_get_sidebar_width_fraction org.gtk.Property.set=adw_navigation_split_view_set_sidebar_width_fraction)
+   * AdwNavigationSplitView:sidebar-width-fraction:
    *
    * The preferred sidebar width as a fraction of the total width.
    *
@@ -848,7 +877,7 @@ adw_navigation_split_view_class_init (AdwNavigationSplitViewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationSplitView:sidebar-width-unit: (attributes org.gtk.Property.get=adw_navigation_split_view_get_sidebar_width_unit org.gtk.Property.set=adw_navigation_split_view_set_sidebar_width_unit)
+   * AdwNavigationSplitView:sidebar-width-unit:
    *
    * The length unit for minimum and maximum sidebar widths.
    *
@@ -931,7 +960,7 @@ adw_navigation_split_view_new (void)
 }
 
 /**
- * adw_navigation_split_view_get_sidebar: (attributes org.gtk.Method.get_property=sidebar)
+ * adw_navigation_split_view_get_sidebar:
  * @self: a navigation split view
  *
  * Gets the sidebar widget for @self.
@@ -949,7 +978,7 @@ adw_navigation_split_view_get_sidebar (AdwNavigationSplitView *self)
 }
 
 /**
- * adw_navigation_split_view_set_sidebar: (attributes org.gtk.Method.set_property=sidebar)
+ * adw_navigation_split_view_set_sidebar:
  * @self: a navigation split view
  * @sidebar: (nullable): the sidebar widget
  *
@@ -1024,7 +1053,7 @@ adw_navigation_split_view_set_sidebar (AdwNavigationSplitView *self,
 }
 
 /**
- * adw_navigation_split_view_get_content: (attributes org.gtk.Method.get_property=content)
+ * adw_navigation_split_view_get_content:
  * @self: a navigation split view
  *
  * Sets the content widget for @self.
@@ -1042,7 +1071,7 @@ adw_navigation_split_view_get_content (AdwNavigationSplitView *self)
 }
 
 /**
- * adw_navigation_split_view_set_content: (attributes org.gtk.Method.set_property=content)
+ * adw_navigation_split_view_set_content:
  * @self: a navigation split view
  * @content: (nullable): the content widget
  *
@@ -1117,7 +1146,7 @@ adw_navigation_split_view_set_content (AdwNavigationSplitView *self,
 }
 
 /**
- * adw_navigation_split_view_get_collapsed: (attributes org.gtk.Method.get_property=collapsed)
+ * adw_navigation_split_view_get_collapsed:
  * @self: a navigation split view
  *
  * Gets whether @self is collapsed.
@@ -1135,7 +1164,7 @@ adw_navigation_split_view_get_collapsed (AdwNavigationSplitView *self)
 }
 
 /**
- * adw_navigation_split_view_set_collapsed: (attributes org.gtk.Method.set_property=collapsed)
+ * adw_navigation_split_view_set_collapsed:
  * @self: a navigation split view
  * @collapsed: whether @self is collapsed
  *
@@ -1169,7 +1198,7 @@ adw_navigation_split_view_set_collapsed (AdwNavigationSplitView *self,
 }
 
 /**
- * adw_navigation_split_view_get_show_content: (attributes org.gtk.Method.get_property=show-content)
+ * adw_navigation_split_view_get_show_content:
  * @self: a navigation split view
  *
  * Gets which page is visible when @self is collapsed.
@@ -1187,7 +1216,7 @@ adw_navigation_split_view_get_show_content (AdwNavigationSplitView *self)
 }
 
 /**
- * adw_navigation_split_view_set_show_content: (attributes org.gtk.Method.set_property=show-content)
+ * adw_navigation_split_view_set_show_content:
  * @self: a navigation split view
  * @show_content: whether to show content when collapsed
  *
@@ -1232,7 +1261,7 @@ adw_navigation_split_view_set_show_content (AdwNavigationSplitView *self,
 }
 
 /**
- * adw_navigation_split_view_get_min_sidebar_width: (attributes org.gtk.Method.get_property=min-sidebar-width)
+ * adw_navigation_split_view_get_min_sidebar_width:
  * @self: a navigation split view
  *
  * Gets the minimum sidebar width for @self.
@@ -1250,7 +1279,7 @@ adw_navigation_split_view_get_min_sidebar_width (AdwNavigationSplitView *self)
 }
 
 /**
- * adw_navigation_split_view_set_min_sidebar_width: (attributes org.gtk.Method.set_property=min-sidebar-width)
+ * adw_navigation_split_view_set_min_sidebar_width:
  * @self: a navigation split view
  * @width: the minimum width
  *
@@ -1282,7 +1311,7 @@ adw_navigation_split_view_set_min_sidebar_width (AdwNavigationSplitView *self,
 }
 
 /**
- * adw_navigation_split_view_get_max_sidebar_width: (attributes org.gtk.Method.get_property=max-sidebar-width)
+ * adw_navigation_split_view_get_max_sidebar_width:
  * @self: a navigation split view
  *
  * Gets the maximum sidebar width for @self.
@@ -1300,7 +1329,7 @@ adw_navigation_split_view_get_max_sidebar_width (AdwNavigationSplitView *self)
 }
 
 /**
- * adw_navigation_split_view_set_max_sidebar_width: (attributes org.gtk.Method.set_property=max-sidebar-width)
+ * adw_navigation_split_view_set_max_sidebar_width:
  * @self: a navigation split view
  * @width: the maximum width
  *
@@ -1332,7 +1361,7 @@ adw_navigation_split_view_set_max_sidebar_width (AdwNavigationSplitView *self,
 }
 
 /**
- * adw_navigation_split_view_get_sidebar_width_fraction: (attributes org.gtk.Method.get_property=sidebar-width-fraction)
+ * adw_navigation_split_view_get_sidebar_width_fraction:
  * @self: a navigation split view
  *
  * Gets the preferred sidebar width fraction for @self.
@@ -1350,7 +1379,7 @@ adw_navigation_split_view_get_sidebar_width_fraction (AdwNavigationSplitView *se
 }
 
 /**
- * adw_navigation_split_view_set_sidebar_width_fraction: (attributes org.gtk.Method.set_property=sidebar-width-fraction)
+ * adw_navigation_split_view_set_sidebar_width_fraction:
  * @self: a navigation split view
  * @fraction: the preferred width fraction
  *
@@ -1383,7 +1412,7 @@ adw_navigation_split_view_set_sidebar_width_fraction (AdwNavigationSplitView *se
 }
 
 /**
- * adw_navigation_split_view_get_sidebar_width_unit: (attributes org.gtk.Method.get_property=sidebar-width-unit)
+ * adw_navigation_split_view_get_sidebar_width_unit:
  * @self: a navigation split view
  *
  * Gets the length unit for minimum and maximum sidebar widths.
@@ -1401,7 +1430,7 @@ adw_navigation_split_view_get_sidebar_width_unit (AdwNavigationSplitView *self)
 }
 
 /**
- * adw_navigation_split_view_set_sidebar_width_unit: (attributes org.gtk.Method.set_property=sidebar-width-unit)
+ * adw_navigation_split_view_set_sidebar_width_unit:
  * @self: a navigation split view
  * @unit: the length unit
  *
