@@ -45,6 +45,9 @@
  * [property@Toast:priority] determines how it behaves if another toast is
  * already being displayed.
  *
+ * Toast titles use Pango markup by default, set [property@Toast:use-markup] to
+ * `FALSE` if this is unwanted.
+ *
  * [property@Toast:custom-title] can be used to replace the title label with a
  * custom widget.
  *
@@ -154,6 +157,7 @@ struct _AdwToast {
   AdwToastPriority priority;
   guint timeout;
   GtkWidget *custom_title;
+  gboolean use_markup;
 
   AdwToastOverlay *overlay;
 };
@@ -167,7 +171,8 @@ enum {
   PROP_PRIORITY,
   PROP_TIMEOUT,
   PROP_CUSTOM_TITLE,
-  LAST_PROP,
+  PROP_USE_MARKUP,
+  LAST_PROP
 };
 
 static GParamSpec *props[LAST_PROP];
@@ -241,6 +246,9 @@ adw_toast_get_property (GObject    *object,
   case PROP_CUSTOM_TITLE:
     g_value_set_object (value, adw_toast_get_custom_title (self));
     break;
+  case PROP_USE_MARKUP:
+    g_value_set_boolean (value, adw_toast_get_use_markup (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -276,6 +284,9 @@ adw_toast_set_property (GObject      *object,
   case PROP_CUSTOM_TITLE:
     adw_toast_set_custom_title (self, g_value_get_object (value));
     break;
+  case PROP_USE_MARKUP:
+    adw_toast_set_use_markup (self, g_value_get_boolean (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -292,7 +303,7 @@ adw_toast_class_init (AdwToastClass *klass)
   object_class->set_property = adw_toast_set_property;
 
   /**
-   * AdwToast:title: (attributes org.gtk.Property.get=adw_toast_get_title org.gtk.Property.set=adw_toast_set_title)
+   * AdwToast:title:
    *
    * The title of the toast.
    *
@@ -308,7 +319,7 @@ adw_toast_class_init (AdwToastClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToast:button-label: (attributes org.gtk.Property.get=adw_toast_get_button_label org.gtk.Property.set=adw_toast_set_button_label)
+   * AdwToast:button-label:
    *
    * The label to show on the button.
    *
@@ -324,7 +335,7 @@ adw_toast_class_init (AdwToastClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToast:action-name: (attributes org.gtk.Property.get=adw_toast_get_action_name org.gtk.Property.set=adw_toast_set_action_name)
+   * AdwToast:action-name:
    *
    * The name of the associated action.
    *
@@ -338,7 +349,7 @@ adw_toast_class_init (AdwToastClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToast:action-target: (attributes org.gtk.Property.get=adw_toast_get_action_target_value org.gtk.Property.set=adw_toast_set_action_target_value)
+   * AdwToast:action-target: (getter get_action_target_value) (setter set_action_target_value)
    *
    * The parameter for action invocations.
    */
@@ -349,7 +360,7 @@ adw_toast_class_init (AdwToastClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToast:priority: (attributes org.gtk.Property.get=adw_toast_get_priority org.gtk.Property.set=adw_toast_set_priority)
+   * AdwToast:priority:
    *
    * The priority of the toast.
    *
@@ -368,7 +379,7 @@ adw_toast_class_init (AdwToastClass *klass)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToast:timeout: (attributes org.gtk.Property.get=adw_toast_get_timeout org.gtk.Property.set=adw_toast_set_timeout)
+   * AdwToast:timeout:
    *
    * The timeout of the toast, in seconds.
    *
@@ -384,7 +395,7 @@ adw_toast_class_init (AdwToastClass *klass)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToast:custom-title: (attributes org.gtk.Property.get=adw_toast_get_custom_title org.gtk.Property.set=adw_toast_set_custom_title)
+   * AdwToast:custom-title:
    *
    * The custom title widget.
    *
@@ -400,6 +411,20 @@ adw_toast_class_init (AdwToastClass *klass)
                          GTK_TYPE_WIDGET,
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwToast:use-markup:
+   *
+   * Whether to use Pango markup for the toast title.
+   *
+   * See also [func@Pango.parse_markup].
+   *
+   * Since: 1.4
+   */
+  props[PROP_USE_MARKUP] =
+    g_param_spec_boolean ("use-markup", NULL, NULL,
+                          TRUE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   /**
@@ -448,6 +473,7 @@ adw_toast_init (AdwToast *self)
   self->priority = ADW_TOAST_PRIORITY_NORMAL;
   self->timeout = 5;
   self->custom_title = NULL;
+  self->use_markup = TRUE;
 
   g_signal_connect (self, "dismissed", G_CALLBACK (dismissed_cb), self);
 }
@@ -511,7 +537,7 @@ adw_toast_new_format (const char *format,
 }
 
 /**
- * adw_toast_get_title: (attributes org.gtk.Method.get_property=title)
+ * adw_toast_get_title:
  * @self: a toast
  *
  * Gets the title that will be displayed on the toast.
@@ -533,7 +559,7 @@ adw_toast_get_title (AdwToast *self)
 }
 
 /**
- * adw_toast_set_title: (attributes org.gtk.Method.set_property=title)
+ * adw_toast_set_title:
  * @self: a toast
  * @title: a title
  *
@@ -559,8 +585,7 @@ adw_toast_set_title (AdwToast   *self,
 
   adw_toast_set_custom_title (self, NULL);
 
-  g_clear_pointer (&self->title, g_free);
-  self->title = g_strdup (title);
+  g_set_str (&self->title, title);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TITLE]);
 
@@ -568,7 +593,7 @@ adw_toast_set_title (AdwToast   *self,
 }
 
 /**
- * adw_toast_get_button_label: (attributes org.gtk.Method.get_property=button-label)
+ * adw_toast_get_button_label:
  * @self: a toast
  *
  * Gets the label to show on the button.
@@ -584,7 +609,7 @@ adw_toast_get_button_label (AdwToast *self)
 }
 
 /**
- * adw_toast_set_button_label: (attributes org.gtk.Method.set_property=button-label)
+ * adw_toast_set_button_label:
  * @self: a toast
  * @button_label: (nullable): a button label
  *
@@ -602,17 +627,14 @@ adw_toast_set_button_label (AdwToast   *self,
 {
   g_return_if_fail (ADW_IS_TOAST (self));
 
-  if (!g_strcmp0 (self->button_label, button_label))
+  if (!g_set_str (&self->button_label, button_label))
     return;
 
-  g_clear_pointer (&self->button_label, g_free);
-  self->button_label = g_strdup (button_label);
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_BUTTON_LABEL]);
 }
 
 /**
- * adw_toast_get_action_name: (attributes org.gtk.Method.get_property=action-name)
+ * adw_toast_get_action_name:
  * @self: a toast
  *
  * Gets the name of the associated action.
@@ -628,7 +650,7 @@ adw_toast_get_action_name (AdwToast *self)
 }
 
 /**
- * adw_toast_set_action_name: (attributes org.gtk.Method.set_property=action-name)
+ * adw_toast_set_action_name:
  * @self: a toast
  * @action_name: (nullable): the action name
  *
@@ -644,17 +666,14 @@ adw_toast_set_action_name (AdwToast   *self,
 {
   g_return_if_fail (ADW_IS_TOAST (self));
 
-  if (!g_strcmp0 (self->action_name, action_name))
+  if (!g_set_str (&self->action_name, action_name))
     return;
 
-  g_clear_pointer (&self->action_name, g_free);
-  self->action_name = g_strdup (action_name);
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ACTION_NAME]);
 }
 
 /**
- * adw_toast_get_action_target_value: (attributes org.gtk.Method.get_property=action-target)
+ * adw_toast_get_action_target_value: (get-property action-target)
  * @self: a toast
  *
  * Gets the parameter for action invocations.
@@ -670,7 +689,7 @@ adw_toast_get_action_target_value (AdwToast *self)
 }
 
 /**
- * adw_toast_set_action_target_value: (attributes org.gtk.Method.set_property=action-target)
+ * adw_toast_set_action_target_value: (set-property action-target)
  * @self: a toast
  * @action_target: (nullable): the action target
  *
@@ -769,7 +788,7 @@ adw_toast_set_detailed_action_name (AdwToast   *self,
 }
 
 /**
- * adw_toast_get_priority: (attributes org.gtk.Method.get_property=priority)
+ * adw_toast_get_priority:
  * @self: a toast
  *
  * Gets priority for @self.
@@ -785,7 +804,7 @@ adw_toast_get_priority (AdwToast *self)
 }
 
 /**
- * adw_toast_set_priority: (attributes org.gtk.Method.set_property=priority)
+ * adw_toast_set_priority:
  * @self: a toast
  * @priority: the priority
  *
@@ -816,7 +835,7 @@ adw_toast_set_priority (AdwToast         *self,
 }
 
 /**
- * adw_toast_get_timeout: (attributes org.gtk.Method.get_property=timeout)
+ * adw_toast_get_timeout:
  * @self: a toast
  *
  * Gets timeout for @self.
@@ -832,7 +851,7 @@ adw_toast_get_timeout (AdwToast *self)
 }
 
 /**
- * adw_toast_set_timeout: (attributes org.gtk.Method.set_property=timeout)
+ * adw_toast_set_timeout:
  * @self: a toast
  * @timeout: the timeout
  *
@@ -859,7 +878,7 @@ adw_toast_set_timeout (AdwToast *self,
 }
 
 /**
- * adw_toast_get_custom_title: (attributes org.gtk.Method.get_property=custom-title)
+ * adw_toast_get_custom_title:
  * @self: a toast
  *
  * Gets the custom title widget of @self.
@@ -877,7 +896,7 @@ adw_toast_get_custom_title (AdwToast *self)
 }
 
 /**
- * adw_toast_set_custom_title: (attributes org.gtk.Method.set_property=custom-title)
+ * adw_toast_set_custom_title:
  * @self: a toast
  * @widget: (nullable): the custom title widget
  *
@@ -897,6 +916,9 @@ adw_toast_set_custom_title (AdwToast  *self,
   g_return_if_fail (ADW_IS_TOAST (self));
   g_return_if_fail (widget == NULL || GTK_IS_WIDGET (widget));
 
+  if (widget)
+    g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
+
   if (self->custom_title == widget)
     return;
 
@@ -948,3 +970,48 @@ adw_toast_set_overlay (AdwToast        *self,
 
   self->overlay = overlay;
 }
+
+/**
+ * adw_toast_get_use_markup:
+ * @self: a toast
+ *
+ * Gets whether to use Pango markup for the toast title.
+ *
+ * Returns: whether the toast uses markup
+ *
+ * Since: 1.4
+ */
+gboolean
+adw_toast_get_use_markup (AdwToast *self)
+{
+  g_return_val_if_fail (ADW_IS_TOAST (self), FALSE);
+
+  return self->use_markup;
+}
+
+/**
+ * adw_toast_set_use_markup:
+ * @self: a toast
+ * @use_markup: whether to use markup
+ *
+ * Whether to use Pango markup for the toast title.
+ *
+ * See also [func@Pango.parse_markup].
+ *
+ * Since: 1.4
+ */
+void
+adw_toast_set_use_markup (AdwToast *self,
+                          gboolean  use_markup)
+{
+  g_return_if_fail (ADW_IS_TOAST (self));
+
+  use_markup = !!use_markup;
+
+  if (adw_toast_get_use_markup (self) == use_markup)
+    return;
+
+  self->use_markup = use_markup;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_USE_MARKUP]);
+}
